'''
A bunch of useful functions separated from the source.
'''

from kivy.utils import platform
from functools import partial
import subprocess
import webbrowser
import os

# probably only the first two will be in KrySA
image_ext = ['.png', '.jpg', '.gif', '.tiff']


# linux & mac need testing
def create_bind(path):
    '''Create a partial for opening a file in system's default program
    for such file extensions.

    .. versionadded:: 0.5.0
    '''
    if isimage(path):
        if platform == 'macosx':
            return partial(subprocess.call, ('open', path))
        elif platform == 'win':
            return partial(win_system, '"' + path + '"')
        elif platform == 'linux':
            return partial(subprocess.call, ('xdg-open', path))
    else:
        return partial(webbrowser.open, path)


def win_system(arg, *args):
    # just a wrapper, because it threw
    # an error about too many args
    os.system(arg)


def isimage(image):
    '''Compare a file from path with the explicitly listed extensions
    for images: `['.png', '.jpg', '.gif', '.tiff']`.

    :Returns:
       `boolean`

    .. versionadded:: 0.5.0
    '''
    global image_ext

    if platform in ['android', 'ios', 'unknown']:
        return None

    for ext in image_ext:
        if ext in image:
            return True



import unittest

import os
import sys
import time
import os.path as op
from shutil import rmtree
from functools import partial
from kivy.clock import Clock

main_path = op.dirname(op.dirname(op.abspath(__file__)))
sys.path.append(main_path)
from main import KrySA, ErrorPop
from tasks.manipulate import Manipulate

# Py3 fixes
import sys
if sys.version_info[0] >= 3:
    unicode = str
    str = bytes
    xrange = range


class Test(unittest.TestCase):
    def pause(*args):
        time.sleep(0.000001)

    def run_test(self, app, *args):
        Clock.schedule_interval(self.pause, 0.000001)

        # open New -> Project popup, set inputs
        app.root._new_project()
        app.root.savedlg.view.selection = [self.folder, ]
        app.root.savedlg.ids.name.text = 'Test.krysa'
        app.root.savedlg.run([self.folder, ], 'Test.krysa')
        project_folder = op.join(self.path, 'test_folder', 'Test')
        data = op.join(project_folder, 'data')
        results = op.join(project_folder, 'results')

        for x in range(4):
            # open New -> Data popup, set inputs
            app.root._new_data()
            new_data = app.root.wiz_newdata.ids.container.children[0]
            new_data.ids.table_name.text = u'NewData' + unicode(x)
            cols = new_data.ids.columns.children

            # set columns for new data
            range_vals = range(13)
            for _ in range(2):
                new_data.ids.columnadd.dispatch('on_release')
                cols[0].ids.colname.text += unicode(len(cols))
                cols[0].ids.coltype.text = 'INTEGER'
                vals = cols[0].ids.vals.children

                for i in range_vals:
                    cols[0].ids.valadd.dispatch('on_release')
                    vals[0].ids.value.text = unicode(i + 1)

                new_data.ids.columnadd.dispatch('on_release')
                cols[0].ids.colname.text += unicode(len(cols))
                cols[0].ids.coltype.text = 'REAL'
                vals = cols[0].ids.vals.children

                for i in range_vals:
                    cols[0].ids.valadd.dispatch('on_release')
                    num = unicode(i + 1)
                    vals[0].ids.value.text = num + '.' + num * x
            new_data = app.root.wiz_newdata.run()

        # (append type, overwrite, table)
        amount = 1
        options = [('Rows', False, '0'), ('Rows', True, '1'),
                   ('Columns', False, '2'), ('Columns', True, '3')]

        for opt in options:
            # check for max len of coltype's Spinner list
            self.assertTrue(amount < 5)

            taskcls = Manipulate()
            taskcls.manip_append()

            children = app.root_window.children
            for c in reversed(children):
                if 'Task' in unicode(c):
                    index = children.index(c)
            task = children[index]

            # fill the task
            body = task.children[0].children[0].children[0].children
            body[-1].text = 'NewData' + opt[2]
            body[-2].children[0].ids.what.text = opt[0]
            body[-2].children[0].ids.overwrite.active = opt[1]
            col_con = body[-2].children[0].ids.cols_container

            if opt[0] == 'Rows':
                body[-2].children[0].ids.amount.text = unicode(amount)
            elif opt[0] == 'Columns':
                col_con = col_con.children[0]
                addcol = col_con.ids.columnadd
                for z in range(amount):
                    addcol.dispatch('on_release')
                    _col = col_con.ids.columns.children[0]
                    _col.ids.colname.text = 'Column' + unicode(z)
                    _col.ids.coltype.text = _col.ids.coltype.values[-z]

            amount += 1
            body[-3].children[0].dispatch('on_release')

        # get results and test
        tables = app.root.tables
        exp_tables = [u'NewData0', u'NewData1', u'NewData2',
                      u'NewData3', u'NewData0_append_1_rows',
                      u'NewData2_append_3_cols']

        # get table list from Task
        taskcls = Manipulate()
        taskcls.manip_append()

        children = app.root_window.children
        for c in reversed(children):
            if 'Task' in unicode(c):
                index = children.index(c)
        task = children[index]
        body = task.children[0].children[0].children[0].children
        table_values = body[-1].values
        body[-3].children[-1].dispatch('on_release')

        # check table order in tab_list and in Task
        self.assertEqual(table_values, exp_tables)
        self.assertEqual([t[0] for t in tables], exp_tables)

        _tables = [tables[4][1], tables[1][1],
                   tables[5][1], tables[3][1]]
        _exp_labels = [
            [u'Data1 (A)', u'Data2 (B)', u'Data3 (C)', u'Data4 (D)'],
            [u'Data1 (A)', u'Data2 (B)', u'Data3 (C)', u'Data4 (D)'],
            [u'Data1 (A)', u'Data2 (B)', u'Data3 (C)', u'Data4 (D)',
             u'Column0 (E)', u'Column1 (F)', u'Column2 (G)'],
            [u'Data1 (A)', u'Data2 (B)', u'Data3 (C)', u'Data4 (D)',
             u'Column0 (E)', u'Column1 (F)', u'Column2 (G)', u'Column3 (H)']
        ]

        for i, tbl in enumerate(_tables):
            data = tbl.rv.data
            labels = []
            for d in data:
                if 'cell' in d.keys():
                    if 'label' in d['cell']:
                        labels.append(d['text'])
            self.assertEqual(labels, _exp_labels[i])
            # check for zeros/order too?

        # test saving at the end (space in table name test)
        app.root.save_project()
        app.stop()

    def test_tasks_manipulate_append(self):
        self.path = op.dirname(op.abspath(__file__))
        if not op.exists(op.join(self.path, 'test_folder')):
            os.mkdir(op.join(self.path, 'test_folder'))
        else:
            rmtree(op.join(self.path, 'test_folder'))
            os.mkdir(op.join(self.path, 'test_folder'))
        self.folder = op.join(self.path, 'test_folder')

        app = KrySA()
        p = partial(self.run_test, app)
        Clock.schedule_once(p, .000001)
        app.run()
        rmtree(self.folder)

if __name__ == '__main__':
    unittest.main()



import unittest

import os
import sys
import time
import os.path as op
from shutil import rmtree
from functools import partial
from kivy.clock import Clock

main_path = op.dirname(op.dirname(op.abspath(__file__)))
sys.path.append(main_path)
from main import KrySA

# Py3 fixes
if sys.version_info[0] >= 3:
    unicode = str
    str = bytes
    xrange = range


class Test(unittest.TestCase):
    def pause(*args):
        time.sleep(0.000001)

    def run_test(self, app, *args):
        Clock.schedule_interval(self.pause, 0.000001)

        # Run twice, test switching projects on True
        recent_file = op.join(app.user_data_dir, 'recent_projects.krysa')
        for create in range(2):
            if create:
                # Create new project
                app.root._new_project()
                app.root.savedlg.view.selection = [self.folder, ]
                app.root.savedlg.ids.name.text = 'Test.krysa'
                app.root.savedlg.run([self.folder, ], 'Test.krysa')
                project_folder = op.join(self.path, 'test_folder', 'Test')
                data = op.join(project_folder, 'data')
                results = op.join(project_folder, 'results')

                # Write paths
                test_file = op.join(self.path, 'test_Project', 'Test.krysa\n')
                with open(recent_file, 'w') as f:
                    paths = '/just/random/path.krysa\n' * 4 + test_file
                    f.write(paths)

            # Show recent projects
            app.root.recent_projects(app.root)
            self.assertTrue(op.exists(recent_file))
            with open(recent_file) as f:
                lines = f.readlines()

            if create:
                self.assertEqual(len(lines), 5)
            else:
                self.assertEqual(lines, [])

        # Check if correct path and open
        self.assertTrue('test_Project' in lines[-2])
        path = app.root._recent_projects()
        for p in path:
            if 'test_Project' in p:
                path = p
        app.root._open_recent(path)
        app.stop()

    def test_file_recentprojects(self):
        self.path = op.dirname(op.abspath(__file__))
        if not op.exists(op.join(self.path, 'test_folder')):
            os.mkdir(op.join(self.path, 'test_folder'))
        else:
            rmtree(op.join(self.path, 'test_folder'))
            os.mkdir(op.join(self.path, 'test_folder'))
        self.folder = op.join(self.path, 'test_folder')

        app = KrySA()
        rmtree(app.user_data_dir)
        p = partial(self.run_test, app)
        Clock.schedule_once(p, .000001)
        app.run()
        rmtree(app.user_data_dir)
        rmtree(self.folder)

if __name__ == '__main__':
    unittest.main()



import unittest

import os
import sys
import time
import sqlite3
import os.path as op
from shutil import rmtree
from functools import partial
from kivy.clock import Clock

main_path = op.dirname(op.dirname(op.abspath(__file__)))
sys.path.append(main_path)
from main import KrySA, ErrorPop
from tasks.basic import Basic


class Test(unittest.TestCase):
    def pause(*args):
        time.sleep(0.000001)

    def run_test(self, app, *args):
        Clock.schedule_interval(self.pause, 0.000001)

        # open New -> Project popup, set inputs
        app.root._new_project()
        app.root.savedlg.view.selection = [self.folder, ]
        app.root.savedlg.ids.name.text = 'Test.krysa'
        app.root.savedlg.run([self.folder, ], 'Test.krysa')
        project_folder = op.join(self.path, 'test_folder', 'Test')
        data = op.join(project_folder, 'data')
        results = op.join(project_folder, 'results')

        # open New -> Data popup, set inputs
        app.root._new_data()
        new_data = app.root.wiz_newdata.ids.container.children[0]
        new_data.ids.table_name.text = 'NewData'
        cols = new_data.ids.columns.children

        # set columns for new data
        range_vals = range(13)
        for _ in range(2):
            new_data.ids.columnadd.dispatch('on_release')
            cols[0].ids.colname.text += str(len(cols))
            cols[0].ids.coltype.text = 'INTEGER'
            vals = cols[0].ids.vals.children

            for i in range_vals:
                cols[0].ids.valadd.dispatch('on_release')
                vals[0].ids.value.text = str(i + 1)

            new_data.ids.columnadd.dispatch('on_release')
            cols[0].ids.colname.text += str(len(cols))
            cols[0].ids.coltype.text = 'REAL'
            vals = cols[0].ids.vals.children

            for i in range_vals:
                cols[0].ids.valadd.dispatch('on_release')
                num = str(i + 1)
                vals[0].ids.value.text = num + '.' + num
        new_data = app.root.wiz_newdata.run()

        # open Task's popup and get task
        k = '3'
        address = ['A1:D13', 'A1:B2', 'C1:D2',
                   'A12:B13', 'C12:D13', 'B3:C10']
        for addr in address:
            taskcls = Basic()
            taskcls.basic_small()

            children = app.root_window.children
            for c in reversed(children):
                if 'Task' in str(c):
                    index = children.index(c)
            task = children[index]

            # fill the task
            body = task.children[0].children[0].children[0].children
            body[-1].text = 'NewData'
            body[-2].children[0].children[-1].children[0].text = addr
            body[-2].children[0].children[-2].children[0].text = k
            body[-3].children[0].dispatch('on_release')

        # get results and test
        expected = reversed([2, 2, 2, 13, 13, 4])
        results = app.root.ids.results
        skipone = False  # if top padding with widget present
        for c in results.children:
            if 'Widget' in str(c):
                skipone = True
                break

        for i, exp in enumerate(expected):
            i = i + 1 if skipone else i
            # Result -> Page -> container -> result
            result = float(results.children[i].ids.page.children[1].text)
            self.assertEqual(result, exp)
        app.stop()

    def test_tasks_basic_small(self):
        self.path = op.dirname(op.abspath(__file__))
        if not op.exists(op.join(self.path, 'test_folder')):
            os.mkdir(op.join(self.path, 'test_folder'))
        else:
            rmtree(op.join(self.path, 'test_folder'))
            os.mkdir(op.join(self.path, 'test_folder'))
        self.folder = op.join(self.path, 'test_folder')

        app = KrySA()
        p = partial(self.run_test, app)
        Clock.schedule_once(p, .000001)
        app.run()
        rmtree(self.folder)

if __name__ == '__main__':
    unittest.main()



from . import Task, SortLayout, AppendLayout, StandLayout
from kivy.uix.tabbedpanel import TabbedPanelItem
from functools import partial

# Py3 fixes
import sys
if sys.version_info[0] >= 3:
    unicode = str
    str = bytes
    xrange = range


class Manipulate(object):
    '''All :ref:`Task` s categorized as being able to `manipulate` data.
    A result after manipulation is a new data.

    .. versionadded:: 0.3.5
    '''

    def manip_sort(*args):
        '''Open a :mod:`tasks.Task` with a :mod:`tasks.SortLayout` that gets
        from user the table which will be sorted and the type of sorting
        (`Ascending` or `Descending`). Create a new :mod:`main.Table`.

        .. versionadded:: 0.3.5
        '''
        widget = SortLayout()
        task = Task(title='Sort', wdg=widget,
                    call=['Sort', Manipulate.manip_sort])
        task.tablecls = task.app.tablecls
        container = task.ids.container.children[0]
        task.run = partial(Manipulate._manip_sort,
                           task,
                           container.ids.sort_type)
        task.open()

    @staticmethod
    def _manip_sort(task, sort_type, *args):
        sort_type = 'Asc' not in sort_type.text
        from_address = task.from_address(task.tablenum, ':all', extended=True)
        values, cols, rows, labels = from_address

        # get separated cols to sort
        chunks = []
        for x in xrange(0, len(values), rows):
            chunks.append(values[x:x + rows])

        values = []
        for val in chunks:
            values.append(sorted(val, reverse=sort_type))

        # add Table
        table = task.ids.tablesel.text
        table += ' (desc)' if sort_type else ' (asc)'
        tabletab = TabbedPanelItem(text=table)
        task.app.root.ids.tabpanel.add_widget(tabletab)

        values = zip(*values)
        values = [v for vals in values for v in vals]
        task.app.root.tables.append((
            table, task.tablecls(max_cols=cols, max_rows=rows,
                                 pos=task.app.root.pos,
                                 size=task.app.root.size,
                                 values=values, labels=labels)
        ))
        tabletab.content = task.app.root.tables[-1][1]

    def manip_filter(*args):
        '''(Not yet implemented)
        '''

    def manip_append(*args):
        '''Open a :mod:`tasks.Task` with a :mod:`tasks.AppendLayout` that gets
        from user :mod:`main.Table`, type of append and an amount of empty
        rows / cols to append.

        The function either returns a new, altered :mod:`main.Table` of
        selected one, or appends directly to the selected :class:`main.Table`.

        .. versionadded:: 0.3.6
        .. versionchanged:: 0.3.7
            Added overwriting of selected :mod:`main.Table`
        .. versionchanged:: 0.5.5
            Added append for new columns
        '''
        widget = AppendLayout()
        task = Task(title='Append', wdg=widget,
                    call=['Append', Manipulate.manip_append])
        task.tablecls = task.app.tablecls
        container = task.ids.container.children[0]
        task.run = partial(Manipulate._manip_append,
                           task,
                           container.ids.what,
                           container.ids.amount,
                           container.ids.cols_container,
                           container.ids.overwrite)
        task.open()

    @staticmethod
    def _manip_append(task, append_type, amount, container, overwrite, *args):
        append_type = append_type.text
        overwrite = overwrite.active
        amount = int(amount.text) if amount.text else 0

        if append_type == 'Append type':
            raise Exception('No append type was chosen!')

        # Stop the task if no amount (or =0) is specified
        # or if no column is available
        if not amount and not container.children:
            raise Exception('No amount was specified!')

        from_address = task.from_address(task.tablenum, ':all',
                                         extended=True)
        values, cols, rows, labels = from_address
        rows = int(rows)

        # get columns
        chunks = []
        for x in xrange(0, len(values), rows):
            chunks.append(values[x:x + rows])
        if append_type == 'Columns':
            _cols = container.children[0].ids.columns.children
            for c in reversed(_cols):
                labels.append(c.ids.colname.text)
                _type = c.ids.coltype.text
                if _type == 'INTEGER':
                    chunks.extend([[0 for _ in range(rows)]])
                elif _type == 'REAL':
                    chunks.extend([[0.0 for _ in range(rows)]])
                else:
                    chunks.extend([[u'' for _ in range(rows)]])
                amount += 1
                cols += 1

        elif append_type == 'Rows':
            # append to columns a zero value according
            # to their type int(amount)-times
            for r in range(amount):
                for chunk in chunks:
                    if isinstance(chunk[0], int):
                        chunk.append(0)
                    elif isinstance(chunk[0], float):
                        chunk.append(0.0)
                    else:
                        chunk.append(u'')

            # increase row count by new rows
            rows += amount

        # zip chunks to values, flatten values
        values = zip(*chunks)
        values = [v for vals in values for v in vals]

        # add Table
        tab_pos = 0
        table = task.ids.tablesel.text
        tabpanel = task.app.root.ids.tabpanel
        tabpanel_len = len(tabpanel.tab_list)
        if overwrite:
            tab_pos = task.tablenum
            # list of available tabs in Task is 0 -> n (from tables),
            # but tab_list order is reversed in Kivy, therefore
            # reverse the index by going backwards with -1
            # and increase tab_pos, which is only index of
            # value order in spinner
            old_tab = tabpanel.tab_list[-1 - (tab_pos + 1)]
            tabpanel.remove_widget(old_tab)
            tabletab = TabbedPanelItem(text=table)
            tabpanel.add_widget(tabletab, tabpanel_len - 1 - (tab_pos + 1))
        else:
            # if space in name, sql save boom
            if append_type == 'Columns':
                table += u'_append_{}_cols'.format(unicode(amount))
            elif append_type == 'Rows':
                table += u'_append_{}_rows'.format(unicode(amount))
            tabletab = TabbedPanelItem(text=table)
            tabpanel.add_widget(tabletab, 0)

        # make new table
        new_table = (
            table, task.tablecls(max_cols=cols, max_rows=rows,
                                 pos=task.app.root.pos,
                                 size=task.app.root.size,
                                 values=values, labels=labels)
        )

        # place newly created table into tab's content
        if overwrite:
            task.app.root.tables[tab_pos] = new_table
            tabletab.content = task.app.root.tables[tab_pos][1]
        else:
            task.app.root.tables.append(new_table)
            tabletab.content = task.app.root.tables[-1][1]

    def manip_split(*args):
        '''(Not yet implemented)
        '''

    def manip_merge(*args):
        '''(Not yet implemented)
        '''

    def manip_stand(*args):
        '''(Not yet implemented)
        Standardizing specified columns from data according to various
        types of standardiztion.
        '''
        widget = StandLayout()
        task = Task(title='Standardize', wdg=widget,
                    call=['Standardize', Manipulate.manip_stand])
        task.tablecls = task.app.tablecls
        container = task.ids.container.children[0]
        task.run = partial(Manipulate._manip_stand,
                           task)
        task.open()

    @staticmethod
    def _manip_stand(task, *args):
        pass

    names = (('Sort', manip_sort),
             ('_Filter', manip_filter),
             ('Append', manip_append),
             ('_Split', manip_split),   # split into two data(columns, rows)
             ('_Merge', manip_merge),
             ('_Standardize', manip_stand))
